import _ from 'lodash/fp.js';
import fetch from 'node-fetch';
import { maybeShowColTypes, getInsertSqlAndArgs, getDeleteByIdSqlAndArgs, getBulkInsertSqlAndArgs, getUpsertSqlAndArgs, getAuthHeader, } from './crate/util.js';
import _debug from 'debug';
import retry from 'p-retry';
const debug = _debug('mongo2crate:crate');
const defaultConfig = { sqlEndpoint: 'http://localhost:4200/_sql' };
export const crate = (config) => {
    const { sqlEndpoint, auth } = _.defaults(defaultConfig, config);
    const authHeader = auth && getAuthHeader(auth);
    debug('Auth header %O', authHeader);
    const query = (sql, options) => {
        const { args, coltypes = false } = options || {};
        debug('query - sql %s args %O', sql, args);
        return retry(() => fetch(maybeShowColTypes(sqlEndpoint, coltypes), {
            method: 'post',
            body: JSON.stringify({ stmt: sql, ...(args && { args }) }),
            headers: { 'Content-Type': 'application/json', ...authHeader },
        }).then((res) => res.json()));
    };
    const insert = (qualifiedName, record) => {
        const { sql, args } = getInsertSqlAndArgs(qualifiedName, record);
        return query(sql, { args });
    };
    const upsert = (qualifiedName, record, update) => {
        const { sql, args } = getUpsertSqlAndArgs(qualifiedName, record, update);
        return query(sql, { args });
    };
    const deleteById = (qualifiedName, id) => {
        const { sql, args } = getDeleteByIdSqlAndArgs(qualifiedName, id);
        return query(sql, { args });
    };
    const bulkInsert = (qualifiedName, records) => {
        const { sql, args } = getBulkInsertSqlAndArgs(qualifiedName, records);
        debug('bulkInsert - sql %s bulk_args %O', sql, args);
        return retry(() => fetch(sqlEndpoint, {
            method: 'post',
            body: JSON.stringify({ stmt: sql, bulk_args: args }),
            headers: { 'Content-Type': 'application/json', ...authHeader },
        }).then((res) => res.json()));
    };
    return { query, insert, upsert, bulkInsert, deleteById };
};
