import type { Collection } from 'mongodb';
import type { Redis } from 'ioredis';
import mongoChangeStream, { ScanOptions, ChangeStreamOptions } from 'mongochangestream';
import { QueueOptions } from 'prom-utils';
import { Crate, ErrorResult, QueryResult } from './crate.js';
import { ConvertOptions, SyncOptions } from './types.js';
export declare const initSync: (redis: Redis, collection: Collection, crate: Crate, options?: SyncOptions & mongoChangeStream.SyncOptions) => {
    /**
     * Process MongoDB change stream for the given collection.
     */
    processChangeStream: (options?: ChangeStreamOptions) => Promise<{
        start: () => Promise<void>;
        stop: () => Promise<void>;
        restart: () => Promise<void>;
    }>;
    /**
     * Run initial collection scan. `options.batchSize` defaults to 500.
     * Sorting defaults to `_id`.
     */
    runInitialScan: (options?: QueueOptions & ScanOptions) => Promise<{
        start: () => Promise<void>;
        stop: () => Promise<void>;
        restart: () => Promise<void>;
    }>;
    /**
     * Convert the given JSON schema to CrateDB table DDL.
     */
    createTableFromSchema: (jsonSchema: object, options?: ConvertOptions) => Promise<QueryResult | ErrorResult>;
    schemaName: string;
    tableName: string;
    qualifiedName: string;
    emitter: import("eventemitter3")<string | symbol, any>;
    reset: () => Promise<void>;
    getCollectionSchema: (db: import("mongodb").Db) => Promise<mongoChangeStream.JSONSchema>;
    detectSchemaChange: (db: import("mongodb").Db, options?: mongoChangeStream.ChangeOptions | undefined) => Promise<{
        start: () => void;
        stop: () => void;
    }>;
    detectResync: (resyncCheckInterval?: number | undefined) => {
        start: () => void;
        stop: () => void;
    };
    keys: {
        scanCompletedKey: string;
        lastScanIdKey: string;
        changeStreamTokenKey: string;
        schemaKey: string;
        lastChangeProcessedAtKey: string;
        lastScanProcessedAtKey: string;
        resyncKey: string;
    };
};
